# -*- coding: utf-8 -*-


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0039_wydzial_archiwalny'),
    ]

    operations = [
        migrations.AlterField(
            model_name='wydzial',
            name='archiwalny',
            field=models.BooleanField(default=False, help_text=b"Je\xc5\xbceli to pole oznaczone jest jako 'PRAWDA', to jednostki 'dawne' - usuni\xc4\x99te\n        ze struktury uczelni, ale nadal wyst\xc4\x99puj\xc4\x85ce w bazie danych, b\xc4\x99d\xc4\x85 przypisywane do tego wydzia\xc5\x82u\n        przez automatyczne procedury integruj\xc4\x85ce dane z zewn\xc4\x99trznych system\xc3\xb3w informatycznych.\n        <br/><br/>\n        Je\xc5\xbceli wi\xc4\x99cej, ni\xc5\xbc jeden wydzia\xc5\x82 w bazie ma to pole oznaczone jako 'PRAWDA', jednostki\n        przypisywane b\xc4\x99d\xc4\x85 do wydzia\xc5\x82u o ni\xc5\xbcszym numerze unikalnego identyfikatora (ID)."),
        ),
    ]



from unittest.mock import patch

import openpyxl
import pytest
from django.urls import reverse
from flexible_reports.models import (
    DATA_FROM_DATASOURCE,
    Column,
    Datasource,
    ReportElement,
    Table,
)
from flexible_reports.models.report import Report
from model_bakery import baker
from six import BytesIO

from formdefaults.models import FormRepresentation
from nowe_raporty.views import (
    AutorRaportFormView,
    GenerujRaportDlaAutora,
    GenerujRaportDlaJednostki,
    GenerujRaportDlaWydzialu,
    JednostkaRaportFormView,
    WydzialRaportFormView,
)

from django.contrib.contenttypes.models import ContentType

from bpp.models import OpcjaWyswietlaniaField
from bpp.models.autor import Autor
from bpp.models.cache import Rekord
from bpp.models.struktura import Jednostka, Wydzial
from bpp.models.wydawnictwo_ciagle import Wydawnictwo_Ciagle


@pytest.mark.django_db
@pytest.mark.parametrize(
    "view,klass",
    [
        ("autor_generuj", Autor),
        ("jednostka_generuj", Jednostka),
        ("wydzial_generuj", Wydzial),
    ],
)
def test_view_raport_nie_zdefiniowany(generuj_raporty_app, view, klass):
    obj = baker.make(klass)
    v = reverse("nowe_raporty:" + view, args=(obj.pk, 2017, 2017))
    res = generuj_raporty_app.get(v)

    assert "Nie znaleziono definicji" in res.text


@pytest.mark.django_db
@pytest.mark.parametrize(
    "view,klass,report_slug",
    [
        (GenerujRaportDlaAutora, Autor, "raport-autorow"),
        (GenerujRaportDlaJednostki, Jednostka, "raport-jednostek"),
        (GenerujRaportDlaWydzialu, Wydzial, "raport-wydzialow"),
    ],
)
def test_view_raport_zdefiniowany(view, klass, report_slug, rf):
    obj = baker.make(klass)
    v = view(kwargs=dict(od_roku=2017, do_roku=2017))
    v.object = obj
    baker.make(Report, slug=report_slug)

    v.request = rf.get("/")
    ret = v.get_context_data()

    assert ret["report"] is not None

    assert ret["report"].base_queryset.all().count() == 0


@pytest.fixture
def autor(typy_odpowiedzialnosci):
    swoja_jednostka = baker.make(Jednostka, skupia_pracownikow=True)
    obca_jednostka = baker.make(Jednostka, skupia_pracownikow=False)

    autor = baker.make(Autor)

    wc1 = baker.make(Wydawnictwo_Ciagle)
    wc2 = baker.make(Wydawnictwo_Ciagle)

    wc1.dodaj_autora(autor, swoja_jednostka, zapisany_jako="lel")
    wc2.dodaj_autora(autor, obca_jednostka, zapisany_jako="lol", afiliuje=False)

    return autor


@pytest.mark.django_db
def test_rekord_prace_autora(autor):
    assert Rekord.objects.prace_autora(autor).count() == 2


@pytest.mark.django_db
def test_rekord_prace_autora_z_afiliowanych_jednostek(autor):
    assert Rekord.objects.prace_autora_z_afiliowanych_jednostek(autor).count() == 1


def test_GenerujRaportDlaAutora_get_base_queryset(rf):
    with patch.object(
        Rekord.objects, "prace_autora_z_afiliowanych_jednostek"
    ) as prace_z_afiliowanych:
        with patch.object(Rekord.objects, "prace_autora") as prace_autora:
            x = GenerujRaportDlaAutora()
            x.request = rf.get("/", data={"_tzju": "True"})
            x.object = None
            x.get_base_queryset()

            prace_z_afiliowanych.assert_called_once()
            prace_autora.assert_not_called()

    with patch.object(
        Rekord.objects, "prace_autora_z_afiliowanych_jednostek"
    ) as prace_z_afiliowanych:
        with patch.object(Rekord.objects, "prace_autora") as prace_autora:
            x = GenerujRaportDlaAutora()
            x.request = rf.get("/", data={"_tzju": "False"})
            x.object = None
            x.get_base_queryset()

            prace_z_afiliowanych.assert_not_called()
            prace_autora.assert_called_once()


@pytest.mark.django_db
def test_czy_jednostka_form_niewidoczny_dla_anonimow(webtest_app, uczelnia):
    baker.make(Report, slug="raport-jednostek")
    res = webtest_app.get(reverse("nowe_raporty:jednostka_form"))
    assert res.status_code == 302
    assert "login" in res.location


@pytest.mark.django_db
def test_czy_jednostka_form_widoczny_dla_zalogowanych(generuj_raporty_app):
    baker.make(Report, slug="raport-jednostek")
    res = generuj_raporty_app.get(reverse("nowe_raporty:jednostka_form"))
    assert res.status_code == 200


@pytest.mark.django_db
def test_czy_generuj_jednostka_niewidoczny_dla_anonimow(webtest_app, jednostka):
    baker.make(Report, slug="raport-jednostek")
    res = webtest_app.get(
        reverse("nowe_raporty:jednostka_generuj", args=(jednostka.pk, 2018, 2020))
    )
    assert res.status_code == 302
    assert "login" in res.location


@pytest.mark.django_db
def test_czy_generuj_jednostka_widoczny_dla_zalogowanych(
    generuj_raporty_app, jednostka, normal_django_user, grupa_raporty_wyswietlanie
):
    baker.make(Report, slug="raport-jednostek")
    res = generuj_raporty_app.get(
        reverse("nowe_raporty:jednostka_generuj", args=(jednostka.pk, 2018, 2020))
    )
    assert res.status_code == 200


@pytest.mark.django_db
def test_czy_wydzial_form_niewidoczny_dla_anonimow(webtest_app, uczelnia):
    baker.make(Report, slug="raport-wydzialow")
    res = webtest_app.get(reverse("nowe_raporty:wydzial_form"))
    assert res.status_code == 302
    assert "login" in res.location


@pytest.mark.django_db
def test_czy_wydzial_form_widoczny_dla_zalogowanych(generuj_raporty_app):
    baker.make(Report, slug="raport-wydzialow")
    res = generuj_raporty_app.get(reverse("nowe_raporty:wydzial_form"))
    assert res.status_code == 200


@pytest.mark.django_db
def test_czy_generuj_wydzial_niewidoczny_dla_anonimow(webtest_app, wydzial):
    baker.make(Report, slug="raport-wydzialow")
    res = webtest_app.get(
        reverse("nowe_raporty:wydzial_generuj", args=(wydzial.pk, 2018, 2020))
    )
    assert res.status_code == 302
    assert "login" in res.location


@pytest.mark.django_db
def test_czy_generuj_wydzial_widoczny_dla_niezalogowanych(generuj_raporty_app, wydzial):
    baker.make(Report, slug="raport-wydzialow")
    res = generuj_raporty_app.get(
        reverse("nowe_raporty:wydzial_generuj", args=(wydzial.pk, 2018, 2020))
    )
    assert res.status_code == 200


@pytest.mark.django_db
def test_czy_raport_autorow_generuj_i_form_przestrzegaja_ustawien_anonim(
    webtest_app, uczelnia, autor_jan_kowalski
):
    baker.make(Report, slug="raport-autorow")

    urls = [
        reverse("nowe_raporty:autor_generuj", args=(autor_jan_kowalski.pk, 2018, 2020)),
        reverse("nowe_raporty:autor_form"),
    ]

    for url in urls:
        uczelnia.pokazuj_raport_autorow = OpcjaWyswietlaniaField.POKAZUJ_ZALOGOWANYM
        uczelnia.save()

        res = webtest_app.get(url)
        assert res.status_code == 302
        assert "login" in res.location

        uczelnia.pokazuj_raport_autorow = OpcjaWyswietlaniaField.POKAZUJ_ZAWSZE
        uczelnia.save()

        res = webtest_app.get(url)
        assert res.status_code == 200

        uczelnia.pokazuj_raport_autorow = OpcjaWyswietlaniaField.POKAZUJ_NIGDY
        uczelnia.save()

        webtest_app.get(url, status=404)


@pytest.mark.django_db
def test_czy_raport_jednostek_generuj_i_form_przestrzegaja_ustawien_anonim(
    webtest_app, uczelnia, jednostka
):
    baker.make(Report, slug="raport-jednostek")

    urls = [
        reverse("nowe_raporty:jednostka_generuj", args=(jednostka.pk, 2018, 2020)),
        reverse("nowe_raporty:jednostka_form"),
    ]

    for url in urls:
        uczelnia.pokazuj_raport_jednostek = OpcjaWyswietlaniaField.POKAZUJ_ZALOGOWANYM
        uczelnia.save()

        res = webtest_app.get(url)
        assert res.status_code == 302
        assert "login" in res.location

        uczelnia.pokazuj_raport_jednostek = OpcjaWyswietlaniaField.POKAZUJ_ZAWSZE
        uczelnia.save()

        res = webtest_app.get(url)
        assert res.status_code == 200

        uczelnia.pokazuj_raport_jednostek = OpcjaWyswietlaniaField.POKAZUJ_NIGDY
        uczelnia.save()

        webtest_app.get(url, status=404)


@pytest.mark.django_db
def test_czy_raport_wydzialow_generuj_i_form_przestrzegaja_ustawien_anonim(
    webtest_app, uczelnia, wydzial
):
    baker.make(Report, slug="raport-wydzialow")

    urls = [
        reverse("nowe_raporty:wydzial_generuj", args=(wydzial.pk, 2018, 2020)),
        reverse("nowe_raporty:wydzial_form"),
    ]

    for url in urls:
        uczelnia.pokazuj_raport_wydzialow = OpcjaWyswietlaniaField.POKAZUJ_ZALOGOWANYM
        uczelnia.save()

        res = webtest_app.get(url)
        assert res.status_code == 302
        assert "login" in res.location

        uczelnia.pokazuj_raport_wydzialow = OpcjaWyswietlaniaField.POKAZUJ_ZAWSZE
        uczelnia.save()

        res = webtest_app.get(url)
        assert res.status_code == 200

        uczelnia.pokazuj_raport_wydzialow = OpcjaWyswietlaniaField.POKAZUJ_NIGDY
        uczelnia.save()

        webtest_app.get(url, status=404)


@pytest.mark.django_db
def test_czy_raport_autorow_generuj_i_form_przestrzegaja_ustawien_zalogowany(
    generuj_raporty_app, uczelnia, autor_jan_kowalski
):
    baker.make(Report, slug="raport-autorow")

    urls = [
        reverse("nowe_raporty:autor_generuj", args=(autor_jan_kowalski.pk, 2018, 2020)),
        reverse("nowe_raporty:autor_form"),
    ]

    for url in urls:
        uczelnia.pokazuj_raport_autorow = OpcjaWyswietlaniaField.POKAZUJ_ZALOGOWANYM
        uczelnia.save()

        res = generuj_raporty_app.get(url)
        assert res.status_code == 200

        uczelnia.pokazuj_raport_autorow = OpcjaWyswietlaniaField.POKAZUJ_ZAWSZE
        uczelnia.save()

        res = generuj_raporty_app.get(url)
        assert res.status_code == 200

        uczelnia.pokazuj_raport_autorow = OpcjaWyswietlaniaField.POKAZUJ_NIGDY
        uczelnia.save()

        generuj_raporty_app.get(url, status=404)


@pytest.mark.django_db
def test_czy_raport_jednostek_generuj_i_form_przestrzegaja_ustawien_zalogowany(
    generuj_raporty_app, uczelnia, jednostka
):
    baker.make(Report, slug="raport-jednostek")

    urls = [
        reverse("nowe_raporty:jednostka_generuj", args=(jednostka.pk, 2018, 2020)),
        reverse("nowe_raporty:jednostka_form"),
    ]

    for url in urls:
        uczelnia.pokazuj_raport_jednostek = OpcjaWyswietlaniaField.POKAZUJ_ZALOGOWANYM
        uczelnia.save()

        res = generuj_raporty_app.get(url)
        assert res.status_code == 200

        uczelnia.pokazuj_raport_jednostek = OpcjaWyswietlaniaField.POKAZUJ_ZAWSZE
        uczelnia.save()

        res = generuj_raporty_app.get(url)
        assert res.status_code == 200

        uczelnia.pokazuj_raport_jednostek = OpcjaWyswietlaniaField.POKAZUJ_NIGDY
        uczelnia.save()

        generuj_raporty_app.get(url, status=404)


@pytest.mark.django_db
def test_czy_raport_wydzialow_generuj_i_form_przestrzegaja_ustawien_zalogowany(
    generuj_raporty_app, uczelnia, wydzial
):
    baker.make(Report, slug="raport-wydzialow")

    urls = [
        reverse("nowe_raporty:wydzial_generuj", args=(wydzial.pk, 2018, 2020)),
        reverse("nowe_raporty:wydzial_form"),
    ]

    for url in urls:
        uczelnia.pokazuj_raport_wydzialow = OpcjaWyswietlaniaField.POKAZUJ_ZALOGOWANYM
        uczelnia.save()

        res = generuj_raporty_app.get(url)
        assert res.status_code == 200

        uczelnia.pokazuj_raport_wydzialow = OpcjaWyswietlaniaField.POKAZUJ_ZAWSZE
        uczelnia.save()

        res = generuj_raporty_app.get(url)
        assert res.status_code == 200

        uczelnia.pokazuj_raport_wydzialow = OpcjaWyswietlaniaField.POKAZUJ_NIGDY
        uczelnia.save()

        generuj_raporty_app.get(url, status=404)


@pytest.mark.django_db
def test_generowanie_xls(
    uczelnia,
    autor_jan_kowalski,
    client,
    rok,
    jednostka,
    wydawnictwo_ciagle,
    wydawnictwo_zwarte,
    typ_odpowiedzialnosci_autor,
):
    wydawnictwo_ciagle.dodaj_autora(autor_jan_kowalski, jednostka)
    wydawnictwo_zwarte.dodaj_autora(autor_jan_kowalski, jednostka)

    r = baker.make(Report, slug="raport-autorow")
    ds = baker.make(Datasource, dsl_query='tytul_oryginalny = "fa"', distinct=True)

    base_model = ContentType.objects.get_for_model(Rekord)

    t = baker.make(Table, base_model=base_model)
    baker.make(Column, parent=t, attr_name="tytul_oryginalny")

    baker.make(
        ReportElement,
        parent=r,
        data_from=DATA_FROM_DATASOURCE,
        base_model=base_model,
        datasource=ds,
        table=t,
    )

    url = (
        reverse("nowe_raporty:autor_generuj", args=(autor_jan_kowalski.pk, rok, rok))
        + "?_export=xlsx&_tzju=True"
    )
    res = client.get(url)

    # Sprawdź, czy to XLS
    wb = openpyxl.load_workbook(BytesIO(res.content))
    assert len(wb.worksheets) == 1


@pytest.mark.parametrize(
    "url,slug,klass",
    [
        ("nowe_raporty:autor_form", "raport-autorow", AutorRaportFormView),
        ("nowe_raporty:jednostka_form", "raport-jednostek", JednostkaRaportFormView),
        ("nowe_raporty:wydzial_form", "raport-wydzialow", WydzialRaportFormView),
    ],
)
def test_form_defaults_napis_przed_po(uczelnia, admin_client, url, klass, slug):
    NAPIS_PRZED = b"napis przed"
    NAPIS_PO = b"napis po"

    baker.make(Report, slug=slug, title="foobar")

    uczelnia.pokazuj_raport_autorow = OpcjaWyswietlaniaField.POKAZUJ_ZAWSZE
    uczelnia.pokazuj_raport_wydzialow = OpcjaWyswietlaniaField.POKAZUJ_ZAWSZE
    uczelnia.pokazuj_raport_jednostek = OpcjaWyswietlaniaField.POKAZUJ_ZAWSZE
    uczelnia.save()

    res = admin_client.get(reverse(url))
    for n in NAPIS_PO, NAPIS_PRZED:
        assert n not in res.content

    res = FormRepresentation.objects.get_or_create_for_instance(klass.form_class())
    res.html_before = NAPIS_PRZED
    res.html_after = NAPIS_PO
    res.save()

    res = admin_client.get(reverse(url))
    for n in NAPIS_PO, NAPIS_PRZED:
        assert n in res.content


def test_GenerujRaportDlaAutora_get_context_data_ukryj_statusy(
    rf, uczelnia, przed_korekta
):
    Report.objects.create(slug="raport-autorow", title="XXX")

    uczelnia.ukryj_status_korekty_set.create(status_korekty=przed_korekta)

    x = GenerujRaportDlaAutora(kwargs=dict(od_roku=0, do_roku=3000))
    x.request = rf.get("/", data={"_tzju": "True"})
    x.object = None
    res = x.get_context_data()
    query = str(res["report"].base_queryset.query)
    assert """AND NOT ("bpp_rekord_mat"."status_korekty_id" IN """ in query


def test_invalid_character_slash_sheet_title(
    autor_jan_kowalski, rf, admin_client, raport_autorow
):
    # Request URL: http://bpp.umlub.pl/nowe_raporty/autor/8553/2020/2020/?_export=xlsx&_tzju=True
    url = reverse(
        "nowe_raporty:autor_generuj", args=(autor_jan_kowalski.pk, 2020, 2020)
    )
    assert admin_client.get(url, data={"_tzju": "True", "_export": "xlsx"})  # noqa



# Generated by Django 2.1.7 on 2019-03-24 18:06

from django.db import migrations, models
import django_fsm


class Migration(migrations.Migration):

    dependencies = [
        ('import_dyscyplin', '0013_auto_20190324_1826'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='kolumna',
            options={'ordering': ['kolejnosc']},
        ),
        migrations.AddField(
            model_name='import_dyscyplin',
            name='wiersz_naglowka',
            field=models.PositiveSmallIntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='kolumna',
            name='kolejnosc',
            field=models.PositiveSmallIntegerField(default=0),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='import_dyscyplin',
            name='stan',
            field=django_fsm.FSMField(choices=[('nowy', 'nowy'), ('określanie opcji importu', 'określanie opcji importu'), ('przeanalizowany', 'przeanalizowany'), ('błędny', 'błędny'), ('zintegrowany', 'zintegrowany')], default='nowy', max_length=50, protected=True),
        ),
    ]



# -*- encoding: utf-8 -*-
from django.core.management import BaseCommand
from django.db import transaction

from bpp.models.struktura import Jednostka, Uczelnia


class Command(BaseCommand):
    help = "Ustawia obcą jednostkę"

    def add_arguments(self, parser):
        parser.add_argument("nazwa")
        parser.add_argument(
            "--wymuszaj",
            default=False,
            action="store_true",
            help="jeżeli PRAWDA to wymusi ustawienie wartości "
            "skupia_pracownikow jednostki na 'fałsz'",
        )

    @transaction.atomic
    def handle(self, nazwa, wymuszaj, *args, **options):
        j = Jednostka.objects.get(nazwa=nazwa)
        u = Uczelnia.objects.first()
        u.obca_jednostka = j
        if wymuszaj:
            if j.skupia_pracownikow:
                j.skupia_pracownikow = False
                j.save()

        u.save()

        j.skupia_pracownikow = False
        j.save()



# -*- coding: utf-8 -*-


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0037_auto_20160124_1336'),
    ]

    operations = [
        migrations.AddField(
            model_name='autor',
            name='pesel_md5',
            field=models.CharField(help_text=b'Hash MD5 numeru PESEL', max_length=32, null=True, db_index=True, blank=True),
        ),
    ]



# Generated by Django 3.0.14 on 2021-08-13 23:03

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0284_auto_20210814_0023"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="bppmultiseekvisibility",
            options={
                "ordering": ("sort_order",),
                "verbose_name": "widoczność opcji wyszukiwania",
                "verbose_name_plural": "widoczność opcji wyszukiwania",
            },
        ),
        migrations.RenameField(
            model_name="bppmultiseekvisibility",
            old_name="name",
            new_name="field_name",
        ),
    ]



# Generated by Django 3.0.14 on 2022-07-06 21:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0322_oplaty_za_publikacje"),
    ]

    operations = [
        migrations.AddField(
            model_name="jednostka",
            name="pokazuj_opis",
            field=models.BooleanField(
                default=True,
                help_text="Gdy to pole jest zaznaczone, system wyświetli pole 'Opis' na podstronie jednostki.",
            ),
        ),
    ]



# Generated by Django 3.0.11 on 2021-03-23 00:06

import datetime

from django.db import migrations, models

from django.utils.timezone import utc


class Migration(migrations.Migration):

    dependencies = [
        ("rozbieznosci_if", "0001_initial"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="ignorujrozbieznoscif",
            options={"verbose_name": "ignorowanie rozbieżności IF"},
        ),
        migrations.AddField(
            model_name="ignorujrozbieznoscif",
            name="created_on",
            field=models.DateTimeField(
                auto_created=True,
                default=datetime.datetime(2021, 3, 23, 0, 6, 43, 174113, tzinfo=utc),
            ),
            preserve_default=False,
        ),
    ]



from django import forms
from django.core.exceptions import ValidationError

from pbn_api.admin.base import BaseMongoDBAdmin
from pbn_api.admin.filters import OdpowiednikWBPPFilter
from pbn_api.admin.helpers import format_json
from pbn_api.models import Publication

from django.contrib import admin

from bpp.models import Uczelnia


class PublicationFromMongoIdForm(forms.ModelForm):
    # mongoId = forms.CharField(max_length=32)

    def __init__(self, *args, **kwargs):
        self.request = kwargs.pop("request")
        super().__init__(*args, **kwargs)

    class Meta:
        model = Publication
        fields = ["mongoId"]

    def clean_mongoId(self):
        uczelnia = Uczelnia.objects.get_for_request(self.request)
        client = uczelnia.pbn_client(self.request.user.pbn_token)
        try:
            self.cleaned_data["json"] = client.get_publication_by_id(
                self.cleaned_data["mongoId"]
            )
        except Exception as e:
            raise ValidationError(
                f"Wystąpił błąd po stronie PBN podczas pobierania danych: {e}"
            )

        return self.cleaned_data["mongoId"]


@admin.register(Publication)
class PublicationAdmin(BaseMongoDBAdmin):
    show_full_result_count = False

    list_display = [
        "title",
        "type",
        "volume",
        "year",
        "publicUri",
        "doi",
        "rekord_w_bpp",
    ]
    search_fields = [
        "mongoId",
        "title",
        "isbn",
        "doi",
        "publicUri",
    ]

    fields = BaseMongoDBAdmin.readonly_fields + [
        "pretty_json",
    ]

    list_filter = [OdpowiednikWBPPFilter] + BaseMongoDBAdmin.list_filter

    def pretty_json(self, obj=None):
        return format_json(obj, "versions")

    pretty_json.short_description = "Odebrane dane (versions)"

    def has_add_permission(self, request):
        return True

    def get_fields(self, request, obj=None):
        if obj is None:
            return ["mongoId"]
        return super(PublicationAdmin, self).get_fields(request, obj)

    def get_readonly_fields(self, request, obj=None):
        if obj is None:
            return []
        return super(PublicationAdmin, self).get_readonly_fields(request, obj)

    def get_form(self, request, obj=None, change=False, **kwargs):
        if not change:

            class RequestPublicationFromMongoIdForm(PublicationFromMongoIdForm):
                def __new__(cls, *args, **kwargs):
                    kwargs["request"] = request
                    return PublicationFromMongoIdForm(*args, **kwargs)

            return RequestPublicationFromMongoIdForm

        return super(PublicationAdmin, self).get_form(
            request=request, obj=obj, change=change, **kwargs
        )

    def save_model(self, request, obj, form, change):
        """
        Given a model instance save it to the database.
        """
        from pbn_api.integrator import zapisz_mongodb

        zapisz_mongodb(form.cleaned_data["json"], Publication)
        obj.refresh_from_db()



# Generated by Django 2.1.7 on 2019-07-18 20:44

from django.db import migrations

from bpp.migration_util import load_custom_sql


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0167_dyscyplina_change_trigger_fix'),
    ]

    operations = [
        migrations.RunPython(
            lambda *args, **kw: load_custom_sql("0168_dyscyplina_w_autorzy_mat")
        ),

    ]



import pytest
from django.core.exceptions import ValidationError

from bpp import const
from bpp.models import (
    Dyscyplina_Naukowa,
    mnoznik_dla_monografii,
    waliduj_format_kodu_numer,
)


def test_Dyscyplina_Naukowa_kod_dziedziny():
    assert Dyscyplina_Naukowa(kod="05.32").kod_dziedziny() == 5


def test_Dyscyplina_Naukowa_dziedzina():
    assert (
        Dyscyplina_Naukowa(kod="03.32").dziedzina()
        == const.DZIEDZINY[const.DZIEDZINA.NAUKI_MEDYCZNE]
    )


@pytest.mark.parametrize(
    "kod,tryb,punktacja,wynik",
    [
        (
            const.DZIEDZINA.NAUKI_TEOLOGICZNE,
            const.TRYB_KALKULACJI.AUTORSTWO_MONOGRAFII,
            200,
            1.5,
        ),
        (
            const.DZIEDZINA.NAUKI_MEDYCZNE,
            const.TRYB_KALKULACJI.AUTORSTWO_MONOGRAFII,
            200,
            1,
        ),
        (
            const.DZIEDZINA.NAUKI_TEOLOGICZNE,
            const.TRYB_KALKULACJI.AUTORSTWO_MONOGRAFII,
            80,
            1.25,
        ),
        (
            const.DZIEDZINA.NAUKI_TEOLOGICZNE,
            const.TRYB_KALKULACJI.AUTORSTWO_MONOGRAFII,
            85,
            1,
        ),
        (
            const.DZIEDZINA.NAUKI_TEOLOGICZNE,
            const.TRYB_KALKULACJI.REDAKCJA_MONOGRAFI,
            100,
            1.5,
        ),
        (
            const.DZIEDZINA.NAUKI_TEOLOGICZNE,
            const.TRYB_KALKULACJI.REDAKCJA_MONOGRAFI,
            90,
            1,
        ),
        (
            const.DZIEDZINA.NAUKI_TEOLOGICZNE,
            const.TRYB_KALKULACJI.ROZDZIAL_W_MONOGRAFI,
            50,
            1.5,
        ),
        (
            const.DZIEDZINA.NAUKI_TEOLOGICZNE,
            const.TRYB_KALKULACJI.ROZDZIAL_W_MONOGRAFI,
            55,
            1,
        ),
    ],
)
def test_moznik_dla_monografii(kod, tryb, punktacja, wynik):
    assert mnoznik_dla_monografii(kod, tryb, punktacja) == wynik


def test_moznik_dla_monografii_error():
    with pytest.raises(NotImplementedError):
        mnoznik_dla_monografii(const.DZIEDZINA.NAUKI_TEOLOGICZNE, 65535, 0)


def test_Dyscyplina_Naukowa_mnoznik_dla_monografii():
    assert Dyscyplina_Naukowa(kod="03.05").mnoznik_dla_monografi(
        const.TRYB_KALKULACJI.AUTORSTWO_MONOGRAFII, 200
    )


def test_waliduj_format_kodu_numer():
    waliduj_format_kodu_numer("1.2")

    with pytest.raises(ValidationError):
        waliduj_format_kodu_numer("a.b")

    with pytest.raises(ValidationError):
        waliduj_format_kodu_numer("500.2")



# -*- encoding: utf-8 -*-


class UML_Egeria_2012_Mangle:
    """Klasa zawierająca informacje, w jaki sposób potraktować nazwy jednostek, ciągi znaków,
    zawarte w importowanym pliku XLS"""

    # Tych jednostek nie importujemy z pliku wejściowego XLS
    ignorowane_jednostki = [
        'Katedra Chemii',
        'Katedra Diagnostyki Laboratoryjnej',
        'Sam. Pracownia Propedeutyki Radiologii Stom. i Szczęk-Twarz',
        'Pracownia Spektrometrii Absorpcji Atomowej',
        'Klinika Endokrynologii i Neurologii Dziecięcej',
        'Pracownia Zarządzania i Ekonomiki Farmaceutycznej',
        'Pracownia Otoneurologiczna',
        'Zakład Elektroradiologii',
        'Pracownia Toksykologii Sądowej',
        'Pracownia Farmakologii Komórkowej i Molekularnej',
        'Pracownia Analizy DNA i Diagnostyki Molekularnej'
        'Samodzielna Pracownia Zdrowia Psychicznego',
        'Zakład Podstaw Pielęgniarstwa i Dydaktyki Medycznej'
    ]

    # W przypadku tych jednostek odpytujemy BEZPOŚREDNIO o nazwę tej jednostki,
    # ponieważ odpytanie standardowe zwróciłoby zbyt dużą ilość jednostek.
    # Ponieważ kliniki mają na początku rzymskie numerki, to icontains
    # znajdzie II katedrę w IIIciej, więc:
    single_fitters = [
        'Zakład Farmakologii',
        'I Katedra i Klinika Ginekologii',
        'II Katedra i Klinika Ginekologii',
        'III Katedra i Klinika Ginekologii',
        ]

    # To zamiany nazw jednostek z tych, które są w XLSie na te, które mamy w BPP
    zamiany_nazw_jednostek = {

        'Katedra i Klinika Ginekologii i Endokrynologii Ginekolog.':
            'Katedra i Klinika Ginekologii i Endokrynologii Ginekologicznej',

        'Zakład Położn., Ginekol. i Pielęg. Położ-Ginekolog.':
            'Zakład Położnictwa, Ginekologii i Pielęgniarstwa Położniczo-Ginekologicznego',

        'Zakład Pielęgniarstwa Anestezjol. i Intensyw. Opieki Medycz.':
            'Zakład Pielęgniarstwa Anestezjologicznego i Intensywnej Opieki Medycznej',

        'Zakład Pedagogiki i Dydak. Medycz. z Prac. Umiejęt. Pielęgn.':
            'Zakład Pedagogiki i Dydaktyki Medycznej z Pracownią Umiejętności Pielęgniarskich',

        'I Klinika Anestezjol. i Intens. Terapii z Klin. Oddz. Dziec.':
            'I Klinika Anestezjologii i Intensywnej Terapii z Klinicznym Oddziałem Dziecięcym',

        'Katedra i Klinika Otolaryn. Dziec., Foniatrii i Audiol.':
            'Katedra i Klinika Otolaryngologii Dziecięcej, Foniatrii i Audiologii',

        'Katedra i Klinika Reumatol. i Układow. Chorób Tkanki Łącznej':
            'Katedra i Klinika Reumatologii i Układowych Chorób Tkanki Łącznej',

        'I Katedra i Klin. Chirur. Ogóln. i Transplant. i Lecz. Żyw.':
            'I Katedra i Klinika Chirurgii Ogólnej, Transplantacyjnej i Leczenia Żywieniowego',

        'II Kated. i Klin. Chir. Ogól. Gastroentero. i Now. Ukł. Pok.':
            'II Katedra i Klinika Chirurgii Ogólnej, Gastroenterologicznej i Nowotworów Układu Pokarmowego',

        'Katedra i Zakł.Mikrob. Farmaceut. z Prac. Diagn. Mikrobiol.':
            'Katedra i Zakład Mikrobiologii Farmaceutycznej z Pracownią Diagnostyki Mikrobiologicznej',

        'Katedra i Zakład Syntezy i Technologii Chem. Środ. Lecznicz.':
            'Katedra i Zakład Syntezy i Technologii Chemicznej Środków Leczniczych',

        'Katedra i Klinika Dermatol., Wenerol. i Dermatol. Dziecięcej':
            'Katedra i Klinika Dermatologii, Wenerologii i Dermatologii Dziecięcej',

        'Zakład Rentgenodiagnostyki Stomatolog. i Szczękowo-Twarzowej':
            'Zakład Rentgenodiagnostyki Stomatologicznej i Szczękowo-Twarzowej',

        'Katedra i Klinika Chirurgii Szczękowo-Twarzowej':
            'Klinika Chirurgii Szczękowo-Twarzowej',

        'Katedra i Zakład Histolog. i Embriol. z Prac. Cytolog. Dośw.':
            'Katedra i Zakład Histologii i Embriologii z Pracownią Cytologii Doświadczalnej',

        'Samodzielna Prawcownia Medycyny Jamy Ustnej':
            'Samodzielna Pracownia Medycyny Jamy Ustnej',

        'Katedra i Zakł. Prot. Stom. z Prac. Zaburz. Czyn. Narz. Żuc.':
            'Zakład Protetyki Stomatologicznej',

        'Klinika Ortopedii i Rehabilitacji':
            'Klinika Ortopedii i Rehabilitacji Katedry Ortopedii',
        }





# Generated by Django 3.0.4 on 2020-03-29 15:19

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("raport_slotow", "0002_auto_20200316_2027"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="raportuczelniaewaluacjaview",
            options={
                "managed": False,
                "ordering": ("rekord__tytul_oryginalny", "autorzy__kolejnosc"),
            },
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import migrations, models
from django.db.migrations.operations.special import RunSQL


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0067_typ_kbn_artykul_pbn'),
    ]

    operations = [
        RunSQL("CREATE INDEX bpp_rekord_mat_liczba_znakow_wydawniczych ON bpp_rekord_mat(liczba_znakow_wydawniczych)",
                "DROP INDEX IF EXISTS bpp_rekord_mat_liczba_znakow_wydawniczych")
    ]



import urllib

from django.db import connection
from django.db.models import CharField
from django.db.models.functions import Cast
from django_filters.views import FilterView
from django_tables2 import SingleTableMixin

from nowe_raporty.views import BaseRaportAuthMixin
from raport_slotow.core import autorzy_zerowi
from raport_slotow.filters import RaportZerowyFilter
from raport_slotow.models import RaportZerowyEntry
from raport_slotow.tables import RaportSlotowZerowyTable
from raport_slotow.util import MyExportMixin, create_temporary_table_as

from django.contrib.postgres.aggregates.general import StringAgg


class RaportSlotowZerowy(
    BaseRaportAuthMixin, MyExportMixin, SingleTableMixin, FilterView
):
    """Pokazuje listę wszystkich autorów, którzy mają zadeklarowane dziedziny naukowe
    ale nie posiadają slotów za konkretny rok.
    """

    template_name = "raport_slotow/raport_slotow_zerowy.html"
    uczelnia_attr = "pokazuj_raport_slotow_zerowy"
    filterset_class = RaportZerowyFilter
    table_class = RaportSlotowZerowyTable

    min_pk = None

    def get_context_data(self, *args, **kwargs):
        context = super().get_context_data(**kwargs)
        context["export_link"] = urllib.parse.urlencode(
            dict(self.request.GET, **{"_export": "xlsx"}), doseq=True
        )
        context["min_pk"] = self.min_pk
        return context

    def get_queryset(self):
        res = autorzy_zerowi(min_pk=self.min_pk)

        create_temporary_table_as("raport_slotow_raportzerowyentry", res)
        with connection.cursor() as cursor:
            cursor.execute(
                "ALTER TABLE raport_slotow_raportzerowyentry ADD COLUMN id SERIAL"
            )

        qset = RaportZerowyEntry.objects.group_by(
            "autor", "dyscyplina_naukowa"
        ).annotate(lata=StringAgg(Cast("rok", CharField()), ", ", ordering=("rok")))

        return qset



# Generated by Django 2.1.10 on 2019-10-27 20:58

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0188_auto_20191027_1737'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='seria_wydawnicza',
            options={'ordering': ('nazwa',), 'verbose_name': 'seria wydawnicza', 'verbose_name_plural': 'serie wydawnicze'},
        ),
    ]



from .conference import *  # noqa
from .institution import *  # noqa
from .journal import *  # noqa
from .monkeypatch_nowrap_foreignkey_admin import *  # noqa
from .oswiadczenieinstytucji import *  # noqa
from .publication import *  # noqa
from .publikacjainstytucji import *  # noqa
from .publisher import *  # noqa
from .scientist import *  # noqa
from .sentdata import *  # noqa



# -*- encoding: utf-8 -*-

from django.apps import AppConfig


class BppConfig(AppConfig):
    name = "bpp"
    verbose_name = "Biblioteka Publikacji Pracowników"

    def ready(self):
        from django.db.models.signals import post_migrate

        from bpp.system import odtworz_grupy, ustaw_robots_txt

        post_migrate.connect(ustaw_robots_txt, sender=self)
        post_migrate.connect(odtworz_grupy, sender=self)



# -*- coding: utf-8 -*-


from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0018_auto_20150824_1233'),
    ]

    operations = [
        migrations.AddField(
            model_name='charakter_formalny',
            name='charakter_pbn',
            field=models.ForeignKey(on_delete=models.CASCADE, default=None, blank=True, to='bpp.Charakter_PBN', null=True),
            preserve_default=True,
        ),
    ]



# Generated by Django 3.2.14 on 2022-07-07 22:32

import django.core.serializers.json
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("import_pracownikow", "0006_importpracownikowrow_tytul"),
    ]

    operations = [
        migrations.AlterField(
            model_name="importpracownikowrow",
            name="dane_z_xls",
            field=models.JSONField(
                blank=True,
                encoder=django.core.serializers.json.DjangoJSONEncoder,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="importpracownikowrow",
            name="dane_znormalizowane",
            field=models.JSONField(
                blank=True,
                encoder=django.core.serializers.json.DjangoJSONEncoder,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="importpracownikowrow",
            name="log_zmian",
            field=models.JSONField(
                blank=True,
                encoder=django.core.serializers.json.DjangoJSONEncoder,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="importpracownikowrow",
            name="podstawowe_miejsce_pracy",
            field=models.BooleanField(null=True),
        ),
    ]



# -*- coding: utf-8 -*-


from django.core.management import call_command
from django.db import migrations, models

from bpp.util import get_fixture


def ustaw_typy_kbn(apps, schema_editor):
    Typ_KBN = apps.get_model("bpp", "Typ_KBN")
    if Typ_KBN.objects.count() == 0:
        data = get_fixture("typ_kbn")
        for elem in data.values():
            Typ_KBN.objects.create(**elem)

    for elem in Typ_KBN.objects.all():
        elem.artykul_pbn = True
        if elem.skrot in ['000', 'PW']:
            elem.artykul_pbn = False
        elem.save()

    Charakter_Formalny = apps.get_model("bpp", "Charakter_Formalny")
    for elem in ["SKR", "PA"]:
        s = Charakter_Formalny.objects.get(skrot=elem)
        s.ksiazka_pbn = True
        s.save()

    for elem in ["ROZS", "frg"]:
        s = Charakter_Formalny.objects.get(skrot=elem)
        s.rozdzial_pbn = True
        s.save()


def noop(apps, schema_editor):
    pass


class Migration(migrations.Migration):
    dependencies = [
        ('bpp', '0066_wydawnictwo_zwarte_calkowita_liczba_autorow'),
    ]

    operations = [
        migrations.AddField(
            model_name='typ_kbn',
            name='artykul_pbn',
            field=models.BooleanField(default=False,
                                      help_text=b'Wydawnictwa ci\xc4\x85g\xc5\x82e posiadaj\xc4\x85ce\n    ten typ KBN zostan\xc4\x85 w\xc5\x82\xc4\x85czone do eksportu PBN jako artyku\xc5\x82y',
                                      verbose_name=b'Artyku\xc5\x82 w PBN'),
        ),
        migrations.RunPython(ustaw_typy_kbn, noop)
    ]



from denorm import denormalized, depend_on_related
from dirtyfields.dirtyfields import DirtyFieldsMixin
from django.db import models
from django.db.models import CASCADE, SET_NULL, JSONField

from django.contrib.postgres.fields import ArrayField

from bpp.models import (
    BazaModeluStreszczen,
    MaProcentyMixin,
    ModelZKwartylami,
    ModelZOplataZaPublikacje,
    parse_informacje,
    wez_zakres_stron,
)
from bpp.models.abstract import (
    BazaModeluOdpowiedzialnosciAutorow,
    DodajAutoraMixin,
    DwaTytuly,
    ModelOpcjonalnieNieEksportowanyDoAPI,
    ModelPunktowany,
    ModelRecenzowany,
    ModelTypowany,
    ModelWybitny,
    ModelZAbsolutnymUrl,
    ModelZAdnotacjami,
    ModelZCharakterem,
    ModelZDOI,
    ModelZeStatusem,
    ModelZeSzczegolami,
    ModelZeZnakamiWydawniczymi,
    ModelZInformacjaZ,
    ModelZISSN,
    ModelZKonferencja,
    ModelZLiczbaCytowan,
    ModelZMiejscemPrzechowywania,
    ModelZNumeremZeszytu,
    ModelZOpenAccess,
    ModelZPBN_UID,
    ModelZPrzeliczaniemDyscyplin,
    ModelZPubmedID,
    ModelZRokiem,
    ModelZWWW,
    Wydawnictwo_Baza,
)
from bpp.models.system import Zewnetrzna_Baza_Danych
from bpp.models.util import ZapobiegajNiewlasciwymCharakterom


class Wydawnictwo_Ciagle_Autor(
    DirtyFieldsMixin,
    BazaModeluOdpowiedzialnosciAutorow,
):
    """Powiązanie autora do wydawnictwa ciągłego."""

    rekord = models.ForeignKey(
        "Wydawnictwo_Ciagle", CASCADE, related_name="autorzy_set"
    )

    class Meta:
        verbose_name = "powiązanie autora z wyd. ciągłym"
        verbose_name_plural = "powiązania autorów z wyd. ciągłymi"
        app_label = "bpp"
        ordering = ("kolejnosc",)
        unique_together = [
            ("rekord", "autor", "typ_odpowiedzialnosci"),
            # Tu musi być autor, inaczej admin nie pozwoli wyedytować
            ("rekord", "autor", "kolejnosc"),
        ]


class ModelZOpenAccessWydawnictwoCiagle(ModelZOpenAccess):
    openaccess_tryb_dostepu = models.ForeignKey(
        "Tryb_OpenAccess_Wydawnictwo_Ciagle",
        SET_NULL,
        verbose_name="OpenAccess: tryb dostępu",
        blank=True,
        null=True,
    )

    class Meta:
        abstract = True


class Wydawnictwo_Ciagle(
    ZapobiegajNiewlasciwymCharakterom,
    Wydawnictwo_Baza,
    DwaTytuly,
    ModelZRokiem,
    ModelZeStatusem,
    ModelZAbsolutnymUrl,
    ModelZWWW,
    ModelZPubmedID,
    ModelZDOI,
    ModelRecenzowany,
    ModelPunktowany,
    ModelTypowany,
    ModelZeSzczegolami,
    ModelZISSN,
    ModelZInformacjaZ,
    ModelZAdnotacjami,
    ModelZCharakterem,
    ModelZOpenAccessWydawnictwoCiagle,
    ModelZeZnakamiWydawniczymi,
    ModelZNumeremZeszytu,
    ModelZKonferencja,
    ModelWybitny,
    ModelZPBN_UID,
    ModelZOplataZaPublikacje,
    ModelZLiczbaCytowan,
    ModelZMiejscemPrzechowywania,
    ModelOpcjonalnieNieEksportowanyDoAPI,
    MaProcentyMixin,
    DodajAutoraMixin,
    DirtyFieldsMixin,
    ModelZPrzeliczaniemDyscyplin,
    ModelZKwartylami,
):
    """Wydawnictwo ciągłe, czyli artykuły z czasopism, komentarze, listy
    do redakcji, publikacje w suplemencie, etc."""

    autor_rekordu_klass = Wydawnictwo_Ciagle_Autor
    autorzy = models.ManyToManyField("Autor", through=autor_rekordu_klass)

    zrodlo = models.ForeignKey(
        "Zrodlo", null=True, verbose_name="Źródło", on_delete=models.SET_NULL
    )

    # To pole nie służy w bazie danych do niczego - jedyne co, to w adminie
    # w wygodny sposób chcemy wyświetlić przycisk 'uzupelnij punktacje', jak
    # się okazuje, przy używaniu standardowych procedur w Django jest to
    # z tego co na dziś dzień umiem, mocno utrudnione.
    uzupelnij_punktacje = models.BooleanField(default=False)

    class Meta:
        verbose_name = "wydawnictwo ciągłe"
        verbose_name_plural = "wydawnictwa ciągłe"
        app_label = "bpp"

    def punktacja_zrodla(self):
        """Funkcja - skrót do użycia w templatkach, zwraca punktację zrodla
        za rok z tego rekordu (self)"""

        from bpp.models.zrodlo import Punktacja_Zrodla

        if hasattr(self, "zrodlo_id") and self.zrodlo_id is not None:
            try:
                return self.zrodlo.punktacja_zrodla_set.get(rok=self.rok)
            except Punktacja_Zrodla.DoesNotExist:
                pass

    def numer_wydania(self):  # issue
        if hasattr(self, "nr_zeszytu"):
            if self.nr_zeszytu:
                return self.nr_zeszytu.strip()

        res = parse_informacje(self.informacje, "numer")
        if res is not None:
            return res.strip()

        return

    def numer_tomu(self):
        if hasattr(self, "tom"):
            if self.tom:
                return self.tom
        return parse_informacje(self.informacje, "tom")

    def zakres_stron(self):
        if self.strony:
            return self.strony
        else:
            strony = wez_zakres_stron(self.szczegoly)
            if strony:
                return strony

    #
    # Cache framework by django-denorm-iplweb
    #

    denorm_always_skip = ("ostatnio_zmieniony",)

    @denormalized(JSONField, blank=True, null=True)
    @depend_on_related(
        "bpp.Wydawnictwo_Ciagle_Autor",
        only=(
            "autor_id",
            "jednostka_id",
            "typ_odpowiedzialnosci_id",
            "afiliuje",
            "dyscyplina_naukowa_id",
            "upowaznienie_pbn",
            "przypieta",
        ),
    )
    def cached_punkty_dyscyplin(self):
        # TODO: idealnie byłoby uzależnić zmiane od pola 'rok' które by było identyczne
        # dla bpp.Poziom_Wydawcy, rok i id z nadrzędnego. Składnia SQLowa ewentualnie
        # jakis zapis django-podobny mile widziany.
        return self.przelicz_punkty_dyscyplin()

    @denormalized(models.TextField, default="")
    @depend_on_related(
        "bpp.Wydawnictwo_Ciagle_Autor",
        only=("zapisany_jako", "typ_odpowiedzialnosci_id", "kolejnosc"),
    )
    @depend_on_related("bpp.Zrodlo", only=("nazwa", "skrot"))
    @depend_on_related("bpp.Charakter_Formalny")
    @depend_on_related("bpp.Typ_KBN")
    @depend_on_related("bpp.Status_Korekty")
    def opis_bibliograficzny_cache(self):
        return self.opis_bibliograficzny()

    @denormalized(ArrayField, base_field=models.TextField(), blank=True, null=True)
    @depend_on_related(
        "bpp.Autor",
        only=(
            "nazwisko",
            "imiona",
        ),
    )
    @depend_on_related("bpp.Wydawnictwo_Ciagle_Autor", only=("kolejnosc",))
    def opis_bibliograficzny_autorzy_cache(self):
        return [
            f"{x.autor.nazwisko} {x.autor.imiona}" for x in self.autorzy_dla_opisu()
        ]

    @denormalized(models.TextField, blank=True, null=True)
    @depend_on_related(
        "bpp.Wydawnictwo_Ciagle_Autor",
        only=("zapisany_jako", "kolejnosc"),
    )
    def opis_bibliograficzny_zapisani_autorzy_cache(self):
        return ", ".join([x.zapisany_jako for x in self.autorzy_dla_opisu()])

    @denormalized(
        models.SlugField,
        max_length=400,
        unique=True,
        db_index=True,
        null=True,
        blank=True,
    )
    @depend_on_related(
        "bpp.Wydawnictwo_Ciagle_Autor",
        only=("zapisany_jako", "kolejnosc"),
    )
    @depend_on_related(
        "bpp.Autor",
        only=("nazwisko", "imiona"),
    )
    @depend_on_related("bpp.Zrodlo", only=("nazwa", "skrot"))
    def slug(self):
        return self.get_slug()

    def clean(self):
        DwaTytuly.clean(self)
        ModelZOplataZaPublikacje.clean(self)


class Wydawnictwo_Ciagle_Zewnetrzna_Baza_Danych(models.Model):
    rekord = models.ForeignKey(
        Wydawnictwo_Ciagle, CASCADE, related_name="zewnetrzna_baza_danych"
    )
    baza = models.ForeignKey(Zewnetrzna_Baza_Danych, CASCADE)
    info = models.CharField(
        verbose_name="Informacje dodatkowe", max_length=512, blank=True, null=True
    )

    def __str__(self):
        return f"{self.baza}"

    class Meta:
        verbose_name = "powiązanie wyd. ciągłego z zewn. bazą danych"
        verbose_name_plural = "powiązania wyd. ciągłych z zewn. bazami danych"


class Wydawnictwo_Ciagle_Streszczenie(BazaModeluStreszczen):
    rekord = models.ForeignKey(Wydawnictwo_Ciagle, CASCADE, related_name="streszczenia")

    class Meta:
        verbose_name = "streszczenie wydawnictwa ciągłego"
        verbose_name_plural = "streszczenia wydawnictw ciągłych"

    def __str__(self):
        if self.jezyk_streszczenia_id is not None:
            return (
                f"Streszczenie rekordu {self.rekord} w języku {self.jezyk_streszczenia}"
            )
        return f"Streszczenie rekordu {self.rekord}"



# Generated by Django 3.0.14 on 2021-10-10 21:08

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0296_nulltest_szablonopisu"),
    ]

    operations = [
        migrations.CreateModel(
            name="Ewaluacja2021LiczbaNDlaUczelni",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("liczba_n", models.DecimalField(decimal_places=4, max_digits=9)),
                (
                    "dyscyplina_naukowa",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="bpp.Dyscyplina_Naukowa",
                    ),
                ),
                (
                    "uczelnia",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="bpp.Uczelnia"
                    ),
                ),
            ],
            options={
                "verbose_name": "Liczba N dla uczelni",
                "verbose_name_plural": "Liczby N dla uczelni",
                "unique_together": {("uczelnia", "dyscyplina_naukowa")},
            },
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import models, migrations

from bpp.fixtures import get_openaccess_data


def utworz_dane_openaccess(apps, schema_editor):
    for model_name, skrot, nazwa in get_openaccess_data():
        klass = apps.get_model('bpp', model_name)
        klass.objects.get_or_create(nazwa=nazwa, skrot=skrot)


def wyrzuc_dane_openaccess(apps, schema_editor):
    for model_name, skrot, nazwa in get_openaccess_data():
        klass = apps.get_model('bpp', model_name)
        klass.get(skrot=skrot).delete()


class Migration(migrations.Migration):
    dependencies = [
        ('bpp', '0027_auto_20150921_2304'),
    ]

    operations = [
        migrations.CreateModel(
            name='Czas_Udostepnienia_OpenAccess',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('nazwa', models.CharField(unique=True, max_length=512)),
                ('skrot', models.CharField(unique=True, max_length=128)),
            ],
            options={
                'ordering': ['nazwa'],
                'verbose_name': 'czas udost\u0119pnienia OpenAccess',
                'verbose_name_plural': 'czasy udost\u0119pnienia OpenAccess',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Licencja_OpenAccess',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('nazwa', models.CharField(unique=True, max_length=512)),
                ('skrot', models.CharField(unique=True, max_length=128)),
            ],
            options={
                'ordering': ['nazwa'],
                'verbose_name': 'licencja OpenAccess',
                'verbose_name_plural': 'licencja OpenAccess',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Tryb_OpenAccess_Wydawnictwo_Ciagle',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('nazwa', models.CharField(unique=True, max_length=512)),
                ('skrot', models.CharField(unique=True, max_length=128)),
            ],
            options={
                'ordering': ['nazwa'],
                'verbose_name': 'tryb OpenAccess wyd. ci\u0105g\u0142ych',
                'verbose_name_plural': 'tryby OpenAccess wyd. ci\u0105g\u0142ych',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Tryb_OpenAccess_Wydawnictwo_Zwarte',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('nazwa', models.CharField(unique=True, max_length=512)),
                ('skrot', models.CharField(unique=True, max_length=128)),
            ],
            options={
                'ordering': ['nazwa'],
                'verbose_name': 'tryb OpenAccess wyd. zwartych',
                'verbose_name_plural': 'tryby OpenAccess wyd. zwartych',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Wersja_Tekstu_OpenAccess',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('nazwa', models.CharField(unique=True, max_length=512)),
                ('skrot', models.CharField(unique=True, max_length=128)),
            ],
            options={
                'ordering': ['nazwa'],
                'verbose_name': 'wersja tekstu OpenAccess',
                'verbose_name_plural': 'wersje tekstu OpenAccess',
            },
            bases=(models.Model,),
        ),
        migrations.AlterField(
            model_name='autor',
            name='pbn_id',
            field=models.IntegerField(null=True, blank=True,
                                      help_text=b'Identyfikator w systemie Polskiej Bibliografii Naukowej (PBN)',
                                      unique=True, verbose_name=b'Identyfikator PBN', db_index=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='jednostka',
            name='pbn_id',
            field=models.IntegerField(null=True, blank=True,
                                      help_text=b'Identyfikator w systemie Polskiej Bibliografii Naukowej (PBN)',
                                      unique=True, verbose_name=b'Identyfikator PBN', db_index=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='uczelnia',
            name='pbn_id',
            field=models.IntegerField(null=True, blank=True,
                                      help_text=b'Identyfikator w systemie Polskiej Bibliografii Naukowej (PBN)',
                                      unique=True, verbose_name=b'Identyfikator PBN', db_index=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='wydzial',
            name='pbn_id',
            field=models.IntegerField(null=True, blank=True,
                                      help_text=b'Identyfikator w systemie Polskiej Bibliografii Naukowej (PBN)',
                                      unique=True, verbose_name=b'Identyfikator PBN', db_index=True),
            preserve_default=True,
        ),
        migrations.RunPython(
            utworz_dane_openaccess, wyrzuc_dane_openaccess
        )
    ]



from model_bakery import baker

from bpp.models import Autor, Charakter_Formalny, Jednostka, Zrodlo
from bpp.models.wydawnictwo_ciagle import Wydawnictwo_Ciagle
from bpp.tests.tests_legacy.testutil import UserRequestFactory, WebTestCase
from bpp.views import autocomplete


class TestAutocomplete(WebTestCase):
    def test_Wydawnictwo_NadrzedneAutocomplete(self):
        Charakter_Formalny.objects.get_or_create(skrot="ROZ", nazwa="Rozdział książki")
        Charakter_Formalny.objects.get_or_create(skrot="ROZS", nazwa="Rozdział skryptu")

        x = autocomplete.Wydawnictwo_NadrzedneAutocomplete()
        x.q = "foobar"
        self.assertTrue(len(x.get_queryset()) is not None)

    def test_JednostkaAutocomplete(self):
        x = autocomplete.JednostkaAutocomplete()
        x.q = "foobar"
        self.assertTrue(len(x.get_queryset()) is not None)

    def test_PublicJednostkaAutocomplete(self):
        x = autocomplete.PublicJednostkaAutocomplete()
        x.q = "foobar"
        self.assertTrue(len(x.get_queryset()) is not None)

    def test_KonferencjaAutocomplete(self):
        x = autocomplete.KonferencjaAutocomplete()
        x.q = "foobar"
        self.assertTrue(len(x.get_queryset()) is not None)

    def test_Seria_WydawniczaAutocomplete(self):
        x = autocomplete.Seria_WydawniczaAutocomplete()
        x.q = "foobar"
        self.assertTrue(len(x.get_queryset()) is not None)

    def test_ZrodloAutocomplete(self):
        x = autocomplete.ZrodloAutocomplete()
        x.q = "foobar"
        self.assertTrue(len(x.get_queryset()) is not None)

    def test_AutorAutocomplete(self):
        x = autocomplete.AutorAutocomplete()
        x.q = "foobar"
        self.assertTrue(len(x.get_queryset()) is not None)
        x.q = ":"
        self.assertTrue(len(x.get_queryset()) is not None)

        self.assertEqual(x.create_object("test").pk, -1)

        res = x.create_object("budnik jan")
        y = Autor.objects.get(pk=res.pk)
        assert y.imiona == "Jan"
        assert y.nazwisko == "Budnik"

        res = x.create_object("kotulowska-papis ilona joanna")
        y = Autor.objects.get(pk=res.pk)
        assert y.imiona == "Ilona Joanna"
        assert y.nazwisko == "Kotulowska-Papis"

    def test_GlobalNavigationAutocomplete(
        self,
    ):
        x = autocomplete.GlobalNavigationAutocomplete()

        class MockUser:
            is_anonymous = False

        x.request = UserRequestFactory(MockUser()).get("/")
        x.q = None
        x.get_result_label(
            "foo"
        )  # XXX uwaga, tu byly 2 parametry, ale test sie wykrzaczał. mpasternak 10.07.2022
        x.get(None)
        x.q = "foobar"
        self.assertTrue(len(x.get_queryset()) is not None)

    def test_GlobalNavigationAutocomplete_query_for_id(self):
        baker.make(Wydawnictwo_Ciagle, pk=123)
        x = autocomplete.GlobalNavigationAutocomplete()
        x.q = "123"
        self.assertTrue(len(x.get_queryset()) == 1)

    def test_GlobalNavigationAutocomplete_test_every_url(self):
        S = "Foobar 123"
        baker.make(Autor, nazwisko=S)
        baker.make(Wydawnictwo_Ciagle, tytul_oryginalny=S)
        baker.make(Zrodlo, nazwa=S)
        baker.make(Jednostka, nazwa=S)

        x = autocomplete.GlobalNavigationAutocomplete()
        x.q = "Foo"
        res = x.get_results({"object_list": list(x.get_queryset())})

        cnt = 0
        for elem in res:
            for child in elem["children"]:
                # Sprawdź, czy global-nav-redir wygeneruje poprawne przekierowanie
                # dla tego zapytania
                url = "/global-nav-redir/%s/" % child["id"]
                res = self.client.get(url)
                self.assertEqual(res.status_code, 302)
                cnt += 1

        self.assertEqual(cnt, 4)

    def test_ZapisanyJakoAutocomplete(self):
        x = autocomplete.ZapisanyJakoAutocomplete()
        a = baker.make(Autor, imiona="Jan", nazwisko="Kowalski", poprzednie_nazwiska="")
        x.forwarded = dict(autor=str(a.id))
        self.assertTrue(len(x.get_list()), 3)

    def test_PodrzednaPublikacjaHabilitacyjnaAutocomplete(self):
        x = autocomplete.PodrzednaPublikacjaHabilitacyjnaAutocomplete()
        a = baker.make(Autor, imiona="Jan", nazwisko="Kowalski", poprzednie_nazwiska="")
        x.forwarded = dict(autor=str(a.id))

        x.q = "foobar"
        self.assertTrue(len(x.get_queryset()) is not None)

    def test_GlobalNavigationAutocomplete_alt(self):
        x = autocomplete.GlobalNavigationAutocomplete()
        x.q = "foobar"
        self.assertTrue(len(x.get_queryset()) is not None)



# Generated by Django 3.2.15 on 2022-08-16 08:19

import uuid

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("zglos_publikacje", "0006_auto_20220815_1752"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="zgloszenie_publikacji_autor",
            options={
                "ordering": ("kolejnosc",),
                "verbose_name": "autor w zgłoszeniu publikacji",
                "verbose_name_plural": "autorzy w zgłoszeniu publikacji",
            },
        ),
        migrations.AddField(
            model_name="zgloszenie_publikacji",
            name="kod_do_edycji",
            field=models.UUIDField(
                blank=True, default=uuid.uuid4, editable=False, null=True, unique=True
            ),
        ),
        migrations.AlterField(
            model_name="zgloszenie_publikacji",
            name="opl_pub_amount",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=20,
                null=True,
                verbose_name="Kwota brutto (zł)",
            ),
        ),
    ]



# Generated by Django 3.0.11 on 2021-02-28 16:39

import django.contrib.postgres.fields.jsonb
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0240_auto_20210228_1739"),
        ("import_pracownikow", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="ImportPracownikowRow",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("wiersz_xls", models.PositiveSmallIntegerField()),
                (
                    "dane_z_xls",
                    django.contrib.postgres.fields.jsonb.JSONField(
                        blank=True, null=True
                    ),
                ),
                (
                    "autor",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="bpp.Autor"
                    ),
                ),
                (
                    "autor_jednostka",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="bpp.Autor_Jednostka",
                    ),
                ),
                (
                    "jednostka",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="bpp.Jednostka"
                    ),
                ),
                (
                    "parent",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="import_pracownikow.ImportPracownikow",
                    ),
                ),
            ],
        ),
    ]



# Generated by Django 3.0.14 on 2021-05-03 22:40

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("pbn_api", "0009_sentdata"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="conference",
            options={
                "verbose_name": "Konferencja w PBN API",
                "verbose_name_plural": "Konferencje w PBN API",
            },
        ),
        migrations.AlterModelOptions(
            name="institution",
            options={
                "verbose_name": "Instytucja w PBN API",
                "verbose_name_plural": "Instytucje w PBN API",
            },
        ),
        migrations.AlterModelOptions(
            name="journal",
            options={
                "verbose_name": "Zródło w PBN API",
                "verbose_name_plural": "Zródła w PBN API",
            },
        ),
        migrations.AlterModelOptions(
            name="publication",
            options={
                "verbose_name": "Publikacja w PBN API",
                "verbose_name_plural": "Publikacje w PBN API",
            },
        ),
        migrations.AlterModelOptions(
            name="publisher",
            options={
                "verbose_name": "Wydawca w PBN API",
                "verbose_name_plural": "Wydawcy w PBN API",
            },
        ),
        migrations.AlterModelOptions(
            name="scientist",
            options={
                "verbose_name": "Osoba w PBN API",
                "verbose_name_plural": "Osoby w PBN API",
            },
        ),
        migrations.AlterModelOptions(
            name="sentdata",
            options={
                "verbose_name": "Informacja o wysłanych danych",
                "verbose_name_plural": "Informacje o wysłanych danych",
            },
        ),
    ]



# Tu skoczyłem
#
# Plan
# 1) zrobić obiekt 'raport uczelnia', zamawianie raportu -> tworzenie go,
# 2) podstrona 'details' to jest:
# - strona po zamowieniu raportu,
# - oczekiwanie na wygenerowanie
# - zamawianie w alternatywnych formatach
# - przejscie do podgladu
#
# 3) listview: raport jest powieszony /rpaort_uczelnia/ID/ i tam jest ListView ewentualnie z filtrami
#
# 4) zadnego progress baru w javie, chociaz nie wiem w sumie
#
from decimal import Decimal

from django.core.exceptions import ValidationError
from django.core.validators import MaxValueValidator
from django.db import models

from bpp.core import zbieraj_sloty
from bpp.fields import YearField
from bpp.models import Autor, Cache_Punktacja_Autora_Query, Uczelnia
from bpp.util import year_last_month
from long_running.models import Report
from long_running.notification_mixins import ASGINotificationMixin
from raport_slotow.core import autorzy_zerowi


class RaportSlotowUczelnia(ASGINotificationMixin, Report):
    od_roku = YearField(default=year_last_month)
    do_roku = YearField(default=Uczelnia.objects.do_roku_default)

    class Akcje(models.TextChoices):
        SLOTY = "slot", "zbieraj najkorzystniejsze prace do zadanej wielkości slotu"
        WSZYSTKO = "wszystko", "wyeksportuj wszystkie prace"

    akcja = models.CharField(max_length=10, default=Akcje.SLOTY, choices=Akcje.choices)

    slot = models.DecimalField(
        default=Decimal("1"),
        decimal_places=4,
        max_digits=7,
        validators=[MaxValueValidator(20)],
        null=True,
        blank=True,
    )

    minimalny_pk = models.DecimalField(default=0, decimal_places=2, max_digits=5)

    dziel_na_jednostki_i_wydzialy = models.BooleanField(
        verbose_name="Dziel na jednostki i wydziały",
        default=True,
    )

    pokazuj_zerowych = models.BooleanField(
        verbose_name="Dołączaj autorów z zerowymi slotami",
        default=False,
        help_text="""Autor z zerowym slotem to autor, który ma na zadany okres czasu zadeklarowane dyscypliny,
        ale nie posiada żadnych punktowanych prac w tych dyscyplinach. Jeżeli jednoczesnie podasz parametr
         'minimalny PK' dla tego raportu, to prace z PK poniżej tego progu zostaną potraktowane jako "zerowe"
         czyli bez punktacji. """,
    )

    def on_reset(self):
        self.raportslotowuczelniawiersz_set.all().delete()

    def clean(self):
        if self.od_roku > self.do_roku:
            raise ValidationError(
                {
                    "od_roku": ValidationError(
                        'Pole musi być większe lub równe jak pole "Do roku".'
                    )
                }
            )

        if self.akcja == RaportSlotowUczelnia.Akcje.WSZYSTKO:
            if self.slot is not None:
                raise ValidationError(
                    {
                        "slot": ValidationError(
                            "Jeżeli chcesz eksportować wszystkie prace, to wielkość slotu nie "
                            "ma znaczenia. Pozostaw to pole puste. "
                        )
                    }
                )

        if self.akcja == RaportSlotowUczelnia.Akcje.SLOTY:
            if self.slot is None:
                raise ValidationError(
                    {
                        "slot": ValidationError(
                            "Jeżeli chcesz zbierać prace do zadanej wielkości slotu, to musisz "
                            "podać wielkość slotu, do którego zbierać prace. "
                        )
                    }
                )

    def create_report(self):
        # lista wszystkich autorow z punktacja z okresu od-do roku
        lst = "autor_id", "dyscyplina_id"
        if self.dziel_na_jednostki_i_wydzialy:
            lst += ("jednostka_id",)

        kombinacje = (
            Cache_Punktacja_Autora_Query.objects.filter(
                rekord__rok__gte=self.od_roku, rekord__rok__lte=self.do_roku
            )
            .values_list(*lst)
            .distinct()
        )

        total = kombinacje.count()

        for n, res in enumerate(kombinacje):
            if self.dziel_na_jednostki_i_wydzialy:
                autor_id, dyscyplina_id, jednostka_id = res
            else:
                autor_id, dyscyplina_id = res
                jednostka_id = None

            maks_punkty, lista, sloty_sum = zbieraj_sloty(
                autor_id,
                self.slot,
                self.od_roku,
                self.do_roku,
                self.minimalny_pk,
                dyscyplina_id=dyscyplina_id,
                jednostka_id=jednostka_id,
                akcja=self.akcja,
            )

            avg = None
            if sloty_sum != 0:
                avg = maks_punkty / sloty_sum

            self.raportslotowuczelniawiersz_set.create(
                autor_id=autor_id,
                jednostka_id=jednostka_id,
                dyscyplina_id=dyscyplina_id,
                slot=sloty_sum,
                pkd_aut_sum=maks_punkty,
                avg=avg,
            )

            if not n % 10:
                self.send_progress(n * 100.0 / total)

        if self.pokazuj_zerowych:
            zerowi = autorzy_zerowi(
                od_roku=self.od_roku, do_roku=self.do_roku, min_pk=self.minimalny_pk
            )

            # --- Początek --- komentarz dot. buga w Django
            # Z uwagi na bug w django (obecnie 3.0.11) nie mozemy zrobic tutaj
            # zerowi = zerowi.values_list("autor_id", "dyscyplina_naukowa_id").distinct()
            # a ja na ten (29. sty 2021) moment nie mam czasu poprawiać Django, zatem uzyjemy
            # set() aby lokalizować autorów już widzianych
            # --- Koniec --- komentarza dot. buga w Django

            # Jeżeli w raporcie jest włączony podział na jednostki i wydziały, to dla każdego autora zerowego
            # dorzuć rekordy jego jednostek; jeżeli nie - to wrzuć po prostu autorów.

            # Kolejny problem: jezeli autor już istnieje w raporcie jako autor który ma prace,
            # to nie powinien być wyświetlany jako zerowy (w kontekscie dyscypliny, w której
            # ma prace).

            seen = set()

            # Dodaj do listy widzianych autorów, którzy już zostali wyemitowani w raporcie dla danych
            # dyscyplin.

            seen = set(
                self.raportslotowuczelniawiersz_set.values_list(
                    "autor_id", "dyscyplina_id"
                ).distinct()
            )

            for autor_id, rok, dyscyplina_id in zerowi.values_list():
                if (autor_id, dyscyplina_id) in seen:
                    continue

                seen.add((autor_id, dyscyplina_id))

                kw = dict(
                    autor_id=autor_id,
                    jednostka_id=None,
                    dyscyplina_id=dyscyplina_id,
                    slot=0,
                    pkd_aut_sum=0,
                    avg=None,
                )

                if self.dziel_na_jednostki_i_wydzialy:
                    for (jednostka_id,) in (
                        Autor.objects.get(pk=autor_id)
                        .autor_jednostka_set.values_list("jednostka_id")
                        .distinct()
                    ):
                        kw["jednostka_id"] = jednostka_id
                        self.raportslotowuczelniawiersz_set.create(**kw)
                else:
                    self.raportslotowuczelniawiersz_set.create(**kw)

    def get_details_set(self):
        return self.raportslotowuczelniawiersz_set.all().select_related(
            "autor", "autor__tytul", "jednostka", "jednostka__wydzial", "dyscyplina"
        )


class RaportSlotowUczelniaWiersz(models.Model):
    parent = models.ForeignKey(RaportSlotowUczelnia, on_delete=models.CASCADE)
    autor = models.ForeignKey("bpp.Autor", on_delete=models.CASCADE)
    jednostka = models.ForeignKey(
        "bpp.Jednostka", on_delete=models.CASCADE, null=True, blank=True
    )
    dyscyplina = models.ForeignKey("bpp.Dyscyplina_Naukowa", on_delete=models.CASCADE)
    pkd_aut_sum = models.DecimalField(
        "Suma punktów dla autora", max_digits=16, decimal_places=4
    )
    slot = models.DecimalField(max_digits=16, decimal_places=4)
    avg = models.DecimalField(
        "Średnio punktów dla autora na slot",
        max_digits=16,
        decimal_places=4,
        null=True,
        blank=True,
    )



# Generated by Django 3.0.4 on 2020-03-29 15:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0207_uczelnia_analiza_view"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="cache_punktacja_autora_query_view",
            options={
                "managed": False,
                "ordering": ("rekord__tytul_oryginalny", "dyscyplina__nazwa"),
            },
        ),
        migrations.AlterField(
            model_name="charakter_formalny",
            name="level",
            field=models.PositiveIntegerField(editable=False),
        ),
        migrations.AlterField(
            model_name="charakter_formalny",
            name="lft",
            field=models.PositiveIntegerField(editable=False),
        ),
        migrations.AlterField(
            model_name="charakter_formalny",
            name="rght",
            field=models.PositiveIntegerField(editable=False),
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import models, migrations


def ustaw_charaktery(apps, schema_editor):
    Charakter_Formalny = apps.get_model('bpp', "Charakter_Formalny")

    mapping = {
        'ROZ': {'rozdzial_pbn': True},
        'ROZS': {'rozdzial_pbn': True},

        'AC': {'artykul_pbn': True},
        'FRG': {'artykul_pbn': True},
        'KOM': {'artykul_pbn': True},
        'L': {'artykul_pbn': True},
        'PRZ': {'artykul_pbn': True},
        'Supl': {'artykul_pbn': True},
        'ZRZ': {'artykul_pbn': True},

        'KS': {'ksiazka_pbn': True},
        'KSZ': {'ksiazka_pbn': True},
        'KSP': {'ksiazka_pbn': True},

    }

    for key, item in list(mapping.items()):
        try:
            obj = Charakter_Formalny.objects.get(skrot=key)
        except Charakter_Formalny.DoesNotExist:  # testy, dalej uzywamy fixtury...
            continue

        for attrname, attrvalue in list(item.items()):
            setattr(obj, attrname, attrvalue)

        obj.save()


class Migration(migrations.Migration):
    dependencies = [
        ('bpp', '0023_auto_20150830_1704'),
    ]

    operations = [
        migrations.RunPython(ustaw_charaktery)
    ]



# Create your models here.
from django.db import models

from long_running.models import Operation


class TestOperation(Operation):
    # long_running.tests.test_models
    pass


class TestObjectThatDoesNotExistManager(models.Manager):
    # long_running.tests.test_tasks
    def get(self, *args, **kw):
        # Those objects never exist
        raise TestObjectThatDoesNotExist.DoesNotExist


class TestObjectThatDoesNotExist(models.Model):
    objects = TestObjectThatDoesNotExistManager()



# Generated by Django 3.0.4 on 2020-04-20 22:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0209_auto_20200421_0013"),
    ]

    operations = [
        migrations.AddField(
            model_name="uczelnia",
            name="sortuj_jednostki_alfabetycznie",
            field=models.BooleanField(
                default=True,
                help_text="Jeżeli ustawione na 'FAŁSZ', sortowanie jednostek będzie odbywało się ręcznie \n        tzn za pomocą ustalonej przez administratora systemu kolejności. ",
            ),
        ),
    ]



from datetime import date, datetime, timedelta

import pytest
from django.db.utils import IntegrityError, InternalError
from model_bakery import baker

from bpp.models.autor import Autor_Jednostka


@pytest.mark.django_db
def test_autor_jednostka_trigger_nie_mozna_zmienic_id_autora(
    autor_jan_kowalski, autor_jan_nowak, jednostka
):
    aj = baker.make(Autor_Jednostka, autor=autor_jan_kowalski, jednostka=jednostka)
    aj.autor = autor_jan_nowak
    with pytest.raises(InternalError):
        aj.save()


@pytest.mark.django_db
def test_autor_jednostka_trigger_nie_mozna_daty_w_przyszlosci(
    autor_jan_kowalski, jednostka
):
    aj = baker.make(Autor_Jednostka, autor=autor_jan_kowalski, jednostka=jednostka)
    aj.zakonczyl_prace = date.today()
    with pytest.raises(IntegrityError):
        aj.save()


@pytest.mark.django_db
def test_autor_jednostka_trigger_ustaw_aktualna_jednostke_1(
    autor_jan_kowalski, jednostka
):
    aj = baker.make(Autor_Jednostka, autor=autor_jan_kowalski, jednostka=jednostka)
    autor_jan_kowalski.refresh_from_db()
    assert autor_jan_kowalski.aktualna_jednostka == jednostka

    aj.zakonczyl_prace = date.today() - timedelta(days=5)
    aj.save()
    autor_jan_kowalski.refresh_from_db()
    assert autor_jan_kowalski.aktualna_jednostka is None

    aj.delete()
    autor_jan_kowalski.refresh_from_db()
    assert autor_jan_kowalski.aktualna_jednostka is None


@pytest.mark.django_db
def test_autor_jednostka_trigger_ustaw_aktualna_jednostke_2(
    autor_jan_kowalski, jednostka, druga_jednostka
):
    baker.make(
        Autor_Jednostka,
        autor=autor_jan_kowalski,
        jednostka=jednostka,
        rozpoczal_prace=date(2012, 1, 1),
        zakonczyl_prace=date(2012, 2, 1),
    )
    baker.make(
        Autor_Jednostka,
        autor=autor_jan_kowalski,
        jednostka=druga_jednostka,
        rozpoczal_prace=date(2012, 1, 1),
        zakonczyl_prace=date(2012, 2, 2),
    )
    autor_jan_kowalski.refresh_from_db()

    assert autor_jan_kowalski.aktualna_jednostka is None


@pytest.mark.django_db
def test_autor_jednostka_trigger_ustaw_aktualna_jednostke_3(
    autor_jan_kowalski, jednostka, druga_jednostka
):
    baker.make(Autor_Jednostka, autor=autor_jan_kowalski, jednostka=druga_jednostka)
    baker.make(Autor_Jednostka, autor=autor_jan_kowalski, jednostka=jednostka)

    # przy dwóch przypisaniach aktualne ma byc to drugie (późniejsze) przypisane
    # sprawdzane jest to po ID w bazie

    autor_jan_kowalski.refresh_from_db()

    assert autor_jan_kowalski.aktualna_jednostka == jednostka


@pytest.mark.django_db
def test_autor_jednostka_trigger_ustaw_aktualna_jednostke_podstawowe_miejsce_pracy(
    autor_jan_kowalski, jednostka, druga_jednostka
):
    baker.make(
        Autor_Jednostka,
        autor=autor_jan_kowalski,
        jednostka=druga_jednostka,
        podstawowe_miejsce_pracy=True,
    )
    baker.make(Autor_Jednostka, autor=autor_jan_kowalski, jednostka=jednostka)

    # aktualne ma byc to miejsce, ktore ma 'podstawowe miejsce pracy' True

    autor_jan_kowalski.refresh_from_db()

    assert autor_jan_kowalski.aktualna_jednostka == druga_jednostka


@pytest.mark.django_db
def test_autor_jednostka_trigger_odepnij_wszystkie_jednostki(
    autor_jan_kowalski, jednostka
):
    baker.make(
        Autor_Jednostka,
        autor=autor_jan_kowalski,
        jednostka=jednostka,
        podstawowe_miejsce_pracy=True,
        zakonczyl_prace=datetime.now().date() - timedelta(days=3),
    )

    autor_jan_kowalski.refresh_from_db()

    # Po odpięciu wszystkiego jednostka ma być None
    assert autor_jan_kowalski.aktualna_jednostka is None



# Register your models here.
from zglos_publikacje.models import Zgloszenie_Publikacji_Autor

from django.contrib.admin import SimpleListFilter

from bpp.models import Wydzial


class WydzialJednostkiPierwszegoAutora(SimpleListFilter):
    title = "wydział 1-go autora"
    parameter_name = "wydz1a"
    db_field_name = "zgloszenie_publikacji_autor__jednostka__wydzial"

    def lookups(self, request, model_admin):
        return [
            (x.pk, x.nazwa)
            for x in Wydzial.objects.filter(
                pk__in=Zgloszenie_Publikacji_Autor.objects.filter(kolejnosc=0)
                .values_list("jednostka__wydzial__pk")
                .distinct()
            )
        ]

    def queryset(self, request, queryset):
        v = self.value()

        field = self.db_field_name
        if field is None:
            field = self.parameter_name

        if v:
            return queryset.filter(
                **{field: v, "zgloszenie_publikacji_autor__kolejnosc": 0}
            )

        return queryset
